# -*- coding: utf-8 -*-
from typing import Union
import argparse

from prolothar_common.models.dataset import TargetSequenceDataset

from experiments.rule_mining.comparison.abstract_comparison import RuleMiningExperiment

from experiments.rule_mining.noise_generator import NoiseGenerator

class RuleMiningExperimentWithVaryingNumberOfInstances(RuleMiningExperiment):

    def __init__(self):
        super().__init__('nr_of_instances')

    def _apply_noise(
            self, noise_generator: NoiseGenerator, trainset: TargetSequenceDataset,
            args, x):
        noise_generator.apply(trainset, args)

    def _add_cli_arguments(self, parser: argparse.ArgumentParser):
        parser.add_argument(
            '--nr_of_instances', type=int, nargs='+',
            help=('list of nr of instances'),
            required=False, default=[100, 200, 300])

    def _run_experiments(self, args):
        for n in args.nr_of_instances:
            self._run_experiment_for_n_instances(n, n, args)

    def _nargs_for_noise_levels(self) -> Union[str, None]:
        return None

if __name__ == '__main__':
    RuleMiningExperimentWithVaryingNumberOfInstances().run_from_cli()
