# -*- coding: utf-8 -*-
from typing import Union
import argparse

from prolothar_common.models.dataset import TargetSequenceDataset

from experiments.rule_mining.comparison.abstract_comparison import RuleMiningExperiment

from experiments.rule_mining.noise_generator import NoiseGenerator

class RuleMiningExperimentWithVaryingNoiseLevel(RuleMiningExperiment):

    def __init__(self):
        super().__init__('noise_level')

    def _apply_noise(
            self, noise_generator: NoiseGenerator, trainset: TargetSequenceDataset,
            args, x):
        noise_generator.apply_ith_noise_level(
            trainset, args, x, logger = self._logger.log)

    def _run_experiments(self, args):
        for i in range(max(len(args.remove_noise), len(args.add_noise), len(args.swap_noise))):
            self._run_experiment_for_n_instances(args.nr_of_instances, i, args)

    def _add_cli_arguments(self, parser: argparse.ArgumentParser):
        parser.add_argument(
            '--nr_of_instances', type=int,
            help=('nr of instances to use in the training set'),
            required=True)

    def _nargs_for_noise_levels(self) -> Union[str, None]:
        return '+'

if __name__ == '__main__':
    RuleMiningExperimentWithVaryingNoiseLevel().run_from_cli()
