# -*- coding: utf-8 -*-
from typing import Union
import argparse

from prolothar_common.models.dataset import TargetSequenceDataset

from experiments.rule_mining.comparison.abstract_comparison import RuleMiningExperiment

from experiments.rule_mining.noise_generator import NoiseGenerator

class RuleMiningExperimentWithVaryingNrOfAttributes(RuleMiningExperiment):

    def __init__(self):
        super().__init__('nr_of_attributes')

    def _apply_noise(
            self, noise_generator: NoiseGenerator, trainset: TargetSequenceDataset,
            args, x):
        noise_generator.apply(trainset, args)

    def _run_experiments(self, args):
        list_of_nr_of_arguments = args.nr_of_attributes
        for nr_of_attributes in list_of_nr_of_arguments:
            args.nr_of_categorical_features = nr_of_attributes // 2
            args.nr_of_numerical_features = nr_of_attributes - args.nr_of_categorical_features
            self._run_experiment_for_n_instances(args.nr_of_instances, nr_of_attributes, args)

    def _add_cli_arguments(self, parser: argparse.ArgumentParser):
        parser.add_argument(
            '--nr_of_instances', type=int,
            help=('nr of instances to use in the training set'),
            required=True)
        parser.add_argument(
            '--nr_of_attributes', type=int,
            help=('nr of attributes to generate'),
            required=True, nargs='+')

    def _nargs_for_noise_levels(self) -> Union[str, None]:
        return None

if __name__ == '__main__':
    RuleMiningExperimentWithVaryingNrOfAttributes().run_from_cli()
