import argparse

from prolothar_common.models.dataset import TargetSequenceDataset
from prolothar_common.models.dataset.transformer import TargetSequenceAddNoise
from prolothar_common.models.dataset.transformer import TargetSequenceRemoveNoise
from prolothar_common.models.dataset.transformer import TargetSequenceSwapNoise

class NoiseGenerator:

    def add_cli_arguments(self, parser: argparse.ArgumentParser, nargs: str = None):
        if nargs is None or nargs == '?':
            default_noise_level = 0.0
        else:
            default_noise_level = []

        parser.add_argument(
                '--remove_noise', type=float, nargs=nargs,
                help='probability of removing events as noise',
                default=default_noise_level, required=False)
        parser.add_argument(
                '--add_noise', type=float, nargs=nargs,
                help='probability of adding events as noise',
                default=default_noise_level, required=False)
        parser.add_argument(
                '--swap_noise', type=float, nargs=nargs,
                help='probability of swapping events as noise',
                default=default_noise_level, required=False)

    def apply(self, dataset: TargetSequenceDataset, args):
        TargetSequenceSwapNoise(
            args.swap_noise, allow_multiple_swaps=False,
            random_seed=args.random_seed).transform(dataset, inplace=True)
        TargetSequenceRemoveNoise(
            args.remove_noise,
            random_seed=args.random_seed).transform(dataset, inplace=True)
        TargetSequenceAddNoise(
            list(dataset.get_set_of_sequence_symbols()), args.add_noise,
            random_seed=args.random_seed).transform(dataset, inplace=True)

    def apply_ith_noise_level(
            self, dataset: TargetSequenceDataset, args, i: int,
            logger = print):
        def get_noise_level(noise_level_list, index):
            try:
                return noise_level_list[index]
            except IndexError:
                return 0.0
        logger('apply noise: (remove: %r, add: %r, swap: %r)' % (
            get_noise_level(args.remove_noise, i),
            get_noise_level(args.add_noise, i),
            get_noise_level(args.swap_noise, i)
        ))
        TargetSequenceSwapNoise(
            get_noise_level(args.swap_noise, i), allow_multiple_swaps=False,
            random_seed=args.random_seed).transform(dataset, inplace=True)
        TargetSequenceRemoveNoise(
            get_noise_level(args.remove_noise, i),
            random_seed=args.random_seed).transform(dataset, inplace=True)
        TargetSequenceAddNoise(
            list(dataset.get_set_of_sequence_symbols()),
            get_noise_level(args.add_noise, i),
            random_seed=args.random_seed).transform(dataset, inplace=True)
