"""
transforms a TargetSequenceDataset to a MultiLabelDataset, i.e. the sequence
prediction task is transformed to the prediction of occurence (yes/no) of events
"""

from prolothar_common.models.dataset import TargetSequenceDataset
from prolothar_common.models.dataset import MultiLabelDataset
from prolothar_common.models.dataset.instance import MultiLabelInstance

class SequenceToMultiLabelDatasetConverter():
    """
    transforms a TargetSequenceDataset to a MultiLabelDataset, i.e. the sequence
    prediction task is transformed to the prediction of occurence (yes/no) of events
    """

    def __init__(self, dataset: TargetSequenceDataset):
        self.__converted_dataset = MultiLabelDataset(
            dataset.get_categorical_attribute_names(),
            dataset.get_numerical_attribute_names()
        )

        for instance in dataset:
            self.__converted_dataset.add_instance(MultiLabelInstance(
                instance.get_id(),
                instance.get_features_dict(),
                set(instance.get_target_sequence())
            ))

    def get_converted_dataset(self) -> MultiLabelDataset:
        return self.__converted_dataset