from typing import List

import heapq

from prolothar_rule_mining.models.event_flow_graph import EventFlowGraph, Node
from prolothar_rule_mining.models.event_flow_graph.alignment.alignment import Alignment
from prolothar_rule_mining.models.event_flow_graph.alignment.heuristics import Heuristic
from prolothar_rule_mining.models.event_flow_graph.alignment.alignment_finder import AlignmentFinder
from prolothar_rule_mining.models.event_flow_graph.alignment.partial_alignment import PartialAlignment

class AStar(AlignmentFinder):

    def __init__(self, graph: EventFlowGraph, heuristic: Heuristic):
        super().__init__(graph)
        self.__heuristic = heuristic

    def compute_alignment(self, sequence: List[str]) -> Alignment:
        openlist = []
        heapq.heappush(openlist, PartialAlignment(
            Alignment(), self.graph.source, 0, 0, len(sequence)+1))

        while openlist:
            current = heapq.heappop(openlist)
            if current.node is self.graph.sink \
            and current.event_index >= len(sequence):
                return current.alignment
            for neighbor in current.yield_neighbors(
                    sequence, self.__heuristic, self.graph.sink):
                heapq.heappush(openlist, neighbor)

        raise NotImplementedError('should never reach this point')
