"""
this module contains the ModelCodeCounter class
"""

from typing import Generator

class ModelCodeCounter:
    """
    counts how many missed, redundant and matched events are present in the cover
    """

    __slots__ = ('missed_events', 'redundant_events', 'matched_events')

    def __init__(self):
        self.missed_events: int = 0
        self.redundant_events: int = 0
        self.matched_events: int = 0

    def __eq__(self, other: 'ModelCodeCounter'):
        return (self.missed_events == other.missed_events
            and self.redundant_events == other.redundant_events
            and self.matched_events == other.matched_events)

    def __repr__(self) -> str:
        return '{missed_events: %d, redundant_events: %d, matched_events: %d}' % (
            self.missed_events, self.redundant_events, self.matched_events
        )

    def __len__(self) -> int:
        """
        this is used in prolothar_common_mdl_utils to compute the prequential
        coding length of this dictionary like structure. "3" is the number of
        different codes (missed, redundant, matched).
        """
        return 3

    def values(self) -> Generator[int,None,None]:
        yield self.missed_events
        yield self.redundant_events
        yield self.matched_events

    def add(self, other: 'ModelCodeCounter'):
        """
        adds the counts of another ModelCodeCounter
        """
        self.missed_events += other.missed_events
        self.redundant_events += other.redundant_events
        self.matched_events += other.matched_events