from typing import Dict, Set

from prolothar_common.models.dataset.instance import Instance
from prolothar_rule_mining.models.event_flow_graph import Node
from prolothar_rule_mining.rule_miner.classification.rules import Rule

class RuledRouter():

    def __init__(self, node_id_to_node: Dict[int, Node], rule: Rule):
        self.__node_id_to_node = node_id_to_node
        self.__rule = rule

    def __call__(self, instance: Instance) -> Node:
        try:
            return self.__node_id_to_node[int(self.__rule.predict(instance))]
        except Exception as e:
            raise NotImplementedError(self.__rule)

    def get_rule(self) -> Rule:
        return self.__rule

    def get_set_of_output_nodes(self) -> Set[Node]:
        return set(
            self.__node_id_to_node[int(node_id)]
            for node_id in self.__rule.get_set_of_output_classes())