from prolothar_common.models.dataset import ClassificationDataset
from prolothar_common.models.dataset.transformer.select_attributes import SelectAttributes
from prolothar_rule_mining.rule_miner.classification.rules import Rule
from prolothar_rule_mining.rule_miner.classification.rules import ListOfRules

from prolothar_rule_mining.fodiscovery.fdd import FoDiscovery
from prolothar_rule_mining.fodiscovery.fdd import GreedyFoDiscovery

class FddRuleMiner():
    """
    first makes a feature selection using a functional dependency analysis and
    then uses a subminer to mine the rules
    """

    def __init__(self, subminer, fo_discovery: FoDiscovery = GreedyFoDiscovery()):
        self.__subminer = subminer
        self.__fo_discovery = fo_discovery

    def mine_rules(self, dataset: ClassificationDataset) -> Rule:
        functional_pattern_list = self.__fo_discovery.run(dataset, 'class')
        #there are no functional dependencies => predict no class
        if not functional_pattern_list:
            return ListOfRules()
        return self.__subminer.mine_rules(
            SelectAttributes(functional_pattern_list[0].attributes).transform(dataset))

    def __repr__(self) -> str:
        return 'FddRuleMiner(%r)' % self.__subminer
