from typing import Set
from abc import ABC, abstractmethod

from prolothar_common.models.dataset.instance import Instance

class Rule(ABC):
    """interface of a classification rule"""

    @abstractmethod
    def predict(self, instance: Instance) -> str:
        """predicts a class for the given instance as input"""

    @abstractmethod
    def to_string(self, prefix='') -> str:
        """returns a human readable string representation of this rule
        Args:
            prefix:
                can be used for indentation
        """

    @abstractmethod
    def to_html(self) -> str:
        """returns a human readable html string representation of this rule"""

    @abstractmethod
    def get_set_of_output_classes(self) -> Set[str]:
        """
        returns the set of classes this rule can return during predict()
        """

    def __str__(self) -> str:
        return self.to_string()

    def __repr__(self) -> str:
        return self.to_string()

    def __hash__(self) -> int:
        return hash(self.to_string())
