from typing import List
from abc import ABC, abstractmethod
from collections import Counter

from prolothar_common.models.dataset import MultisetDataset
from prolothar_common.models.dataset import TargetSequenceDataset

class AbstractOrdering(ABC):
    """
    template of trainable ordering methods for multisets
    """

    @abstractmethod
    def train(self, multiset_dataset: MultisetDataset, sequence_dataset: TargetSequenceDataset):
        """
        learns how to order multisets from a multiset_dataset and a sequence_dataset
        that share the same set of instances (in the same order).

        Parameters
        ----------
        multiset_dataset : MultisetDataset
            contains instances with multiset target
        sequence_dataset : TargetSequenceDataset
            contains the same instances as multiset_dataset but with sequence
            targets
        """

    @abstractmethod
    def order(self, multiset: Counter) -> List:
        """
        orders the given multiset and outputs a list
        """