from typing import Tuple

from prolothar_rule_mining.models.event_flow_graph import EventFlowGraph
from prolothar_rule_mining.models.event_flow_graph.alignment.alignment_finder import AlignmentFinder
from prolothar_rule_mining.models.event_flow_graph.alignment.greedy_shortest_path import GreedyShortestPath
from prolothar_rule_mining.models.event_flow_graph.alignment.heuristics import ReachabilityHeuristic

from prolothar_rule_mining.rule_miner.data_to_sequence.rudi.candidates import Candidate

class AddSequencePathCandidate(Candidate):
    """
    adds a path to the graph to match a given sequence
    """
    def __init__(self, graph: EventFlowGraph, sequence: Tuple[str],
                 alignment_finder: AlignmentFinder = None):
        super().__init__(graph, None)
        if alignment_finder is None:
            self.__alignment_finder = GreedyShortestPath(
                graph, ReachabilityHeuristic(graph))
        else:
            self.__alignment_finder = alignment_finder

        self.__sequence = sequence
        self.__graph = graph

    def _apply(self):
        if self.__graph.get_nr_of_edges() == 0:
            self.__add_path_for_empty_graph()
        else:
            self.__add_path_for_none_empty_graph()

    def __add_path_for_empty_graph(self):
        last_node = self.__graph.source
        for event in self.__sequence:
            new_node = self._add_node(event)
            self._add_edge(last_node, new_node)
            last_node = new_node
        self.__graph.add_edge(last_node, self.__graph.sink)

    def __add_path_for_none_empty_graph(self):
        last_node = self.__graph.source
        for move in self.__alignment_finder.compute_alignment(self.__sequence):
            if move.is_log_move():
                if move.event_index < len(self.__sequence):
                    new_node = self._add_node(self.__sequence[move.event_index])
                else:
                    new_node = self.__graph.sink
                self._add_edge(last_node, new_node)
                last_node = new_node
            elif move.is_sync_move():
                if move.node not in last_node.children:
                    self._add_edge(last_node, move.node)
                last_node = move.node

    def leads_to_cycle(self) -> bool:
        return False