from typing import List

import heapq

from prolothar_rule_mining.rule_miner.data_to_sequence.rudi.candidates.candidate import Candidate


class CandidateQueue:
    """
    a priority queue of the candidates
    """

    def __init__(self):
        self.__queue: List[Candidate] = []

    def add(self, candidate: Candidate):
        """
        adds the candidate to this queue
        """
        heapq.heappush(self.__queue, candidate)

    def pop(self) -> Candidate:
        """
        pops the candidate with the lowest priority
        """
        return heapq.heappop(self.__queue)

    def is_not_empty(self) -> bool:
        """
        returns True if there are still elements in this queue, otherwise False
        """
        return bool(self.__queue)

    def __len__(self) -> int:
        """
        returns the length (the number of candidates) in this queue
        """
        return len(self.__queue)
