"""
this module contains the event flow graph miner, i.e. for a given dataset an
event flow graph is inferred
"""

from typing import Callable

from prolothar_common.models.dataset import TargetSequenceDataset
from prolothar_common.func_tools import do_nothing

from prolothar_rule_mining.models.event_flow_graph import EventFlowGraph
from prolothar_rule_mining.models.event_flow_graph.alignment.petrinet import PetrinetAligner

from prolothar_rule_mining.rule_miner.data_to_sequence.rudi.event_flow_graph_miner.abstract import EventFlowGraphMiner
from prolothar_rule_mining.rule_miner.data_to_sequence.rudi.candidates import AddSequencePathCandidate

class SequenceAlignmentNoMdl(EventFlowGraphMiner):
    """
    similar to SequenceBottomUpEventFlowGraphMiner but without using MDL to
    decide when to stop
    """

    def __init__(self, logger: Callable[[str], None] = print):
        """
        creates a new EventFlowGraphMiner
        """
        self.__logger = logger if logger is not None else do_nothing

    def mine_event_flow_graph(self, dataset: TargetSequenceDataset) -> EventFlowGraphMiner:
        graph = EventFlowGraph()

        all_sequences = dataset.get_sequences_ordered_by_frequency()
        for i,sequence in enumerate(reversed(all_sequences)):
            self.__logger(f'extend model for sequence {i+1} of {len(all_sequences)}')
            AddSequencePathCandidate(
                graph, sequence, alignment_finder=PetrinetAligner(graph)).apply()

        graph.merge_redundant_nodes()

        return graph

    def __repr__(self) -> str:
        return 'SequenceAlignmentNoMdl()'