from typing import Dict, Tuple

from prolothar_rule_mining.rule_miner.classification.rules import Rule

class ClassificationRule():
    """
    a data to sequence rule interface that is based on learned classifactor that
    predicts a class label which corresponds to a sequence in the training set
    """

    def __init__(self, classification_rule: Rule, class_to_sequence: Dict[str, Tuple[str]]):
        self.__classification_rule = classification_rule
        self.__class_to_sequence = class_to_sequence

    def execute(self, instance) -> Tuple[str]:
        """generates a sequence for the given instance as input"""
        return self.__class_to_sequence[self.__classification_rule.predict(instance)]

    def count_nr_of_terms(self) -> int:
        """
        returns the number of terms (atomic conditions).
        "If a > 2 and b <= 3 then append(x)" returns 2
        """
        return self.__classification_rule.count_nr_of_terms()
