from typing import Union, Callable

from prolothar_common.models.dataset import Dataset
from prolothar_rule_mining.rule_miner.classification.rules import Rule as ClassificationRule
from prolothar_rule_mining.rule_miner.data_to_sequence.rules import Rule as DataToSequenceRule
from prolothar_rule_mining.rule_miner.multilabel.rules import Rule as MultilabelRule

Rule = Union[ClassificationRule, DataToSequenceRule, MultilabelRule]

class DatasetSplittingMiner:
    """
    Adapter to use a hold out set training strategy for specific rule miners
    """

    def __init__(
            self, training_function: Callable[[Dataset, Dataset], Rule],
            hold_out_set_ratio: float = 0.2, random_seed: Union[int, None] = None):
        self.__hold_out_set_ratio = hold_out_set_ratio
        self.__random_seed = random_seed
        self.__training_function = training_function

    def mine_rules(self, dataset: Dataset) -> Rule:
        trainset, testset = dataset.split(
            self.__hold_out_set_ratio, random_seed=self.__random_seed)
        return self.__training_function(trainset, testset)
