from typing import Dict

from prolothar_common.models.dataset.instance import Instance
from prolothar_rule_mining.models.conditions import Condition

class BoomerRule():
    """
    a single rule in the model of Boomer.
    """
    def __init__(self, condition: Condition, event_confidence_map: Dict[str, float]):
        self.__condition = condition
        self.__event_confidence_map = event_confidence_map

    def applies_to_instance(self, instance: Instance):
        """
        returns True if the condition of this rule applies to the given instance
        or if the condition is None
        """
        return self.__condition is None or self.__condition.check_instance(instance)

    def add_confidence(self, confidence_per_event: Dict[str, float]):
        """
        add the confidence values of this rule to the given confidence per
        event dictionary
        """
        for event, confidence in self.__event_confidence_map.items():
            confidence_per_event[event] = confidence_per_event.get(
                event, 0) + confidence

    def count_nr_of_terms(self) -> int:
        """
        returns the number of condition terms in this rule
        """
        return 0 if self.__condition is None else self.__condition.count_nr_of_terms()

    def __repr__(self) -> str:
        return f'BoomerRule({self.__condition}, {self.__event_confidence_map})'