import math

class ClassAccuracy:
    def __init__(self):
        """ Initialize the accuracy calculation for a single class """
        self.T_myClass = 0  # For binary class problems this would include true positives
        self.T_otherClass = 0  # For binary class problems this would include true negatives
        self.F_myClass = 0  # For binary class problems this would include false positives
        self.F_otherClass = 0  # For binary class problems this would include false negatives

    def updateAccuracy(self, thisIsMe, accurateClass):
        """ Increment the appropriate cell of the confusion matrix """
        if thisIsMe and accurateClass:
            self.T_myClass += 1
        elif accurateClass:
            self.T_otherClass += 1
        elif thisIsMe:
            self.F_myClass += 1
        else:
            self.F_otherClass += 1

    def reportClassAccuracy(self):
        """ Print to standard out, summary on the class accuracy. """
        print("-----------------------------------------------")
        print("TP = " + str(self.T_myClass))
        print("TN = " + str(self.T_otherClass))
        print("FP = " + str(self.F_myClass))
        print("FN = " + str(self.F_otherClass))
        print("-----------------------------------------------")

    def labelCount(self, phenotypePrediction, true_phenotype):
        self.trueLabelCount = self.countLabel(true_phenotype)
        self.predictedLabelCount = self.countLabel(phenotypePrediction)

    def countLabel(self, phenotype):
        count = 0
        for L in phenotype:
            if float(L) != 0:
                count += 1
        return count

    def intersection(self, phenotypePrediction, true_phenotype):
        self.Intersect = 0
        for (L, L_hat) in zip(phenotypePrediction, true_phenotype):
            if float(L) != 0:
                self.Intersect += float(L)*float(L_hat)

    def union(self, phenotypePrediction, true_phenotype):
        self.Union = 0
        for (L, L_hat) in zip(phenotypePrediction, true_phenotype):
            if (float(L) or float(L_hat)) != 0:
                self.Union += 1

    def precision(self):
        precision = self.Intersect / self.trueLabelCount
        return precision

    def recall(self):
        if self.predictedLabelCount == 0:
            recall = 0.0
        else:
            recall = self.Intersect / self.predictedLabelCount
        return recall

    def accuracy(self):
        accuracy = self.Intersect / self.Union
        return accuracy

    def f_measure(self):
        f_measure = 2*self.Intersect/(self.trueLabelCount + self.predictedLabelCount)
        return f_measure

    def hammingLoss(self, phenotypePrediction, true_phenotype):
        Dist = 0.0
        for (L, L_hat) in zip(phenotypePrediction, true_phenotype):
            Dist += math.fabs((float(L) - float(L_hat)))
        loss = Dist/len(phenotypePrediction)
        return loss

    def oneError(self, combinedVote, true_phenotype):
        maxVote = max(combinedVote)
        maxIndex = []
        for (ind, v) in enumerate(combinedVote):
            if v == maxVote:
                maxIndex.append(ind)
        error = 1
        for ind in maxIndex:
            if true_phenotype[ind] == '1':
                error = 0
                break
        return error

    def rankLoss(self, combinedVote, true_phenotype):
        L = []
        Lbar = []
        for (i, l) in enumerate(true_phenotype):
            if l == '0':
                Lbar.append(i)
            else:
                L.append(i)
        lossCount = 0
        for l in L:
            for lbar in Lbar:
                if combinedVote[l] == 0:
                    lossCount +=1
                elif combinedVote[l] <= combinedVote[lbar]:
                    lossCount += 1
        if len(Lbar)==0:
            lossCount = len(L)
        else:
            lossCount = lossCount/(len(L)*len(Lbar))
        return lossCount

    def multiLablePerformace(self, phenotypePrediction, true_phenotype, combinedVote):
        self.labelCount(phenotypePrediction, true_phenotype)
        self.intersection(phenotypePrediction, true_phenotype)
        self.union(phenotypePrediction, true_phenotype)
        self.loss_single = self.hammingLoss(phenotypePrediction, true_phenotype)
        self.precision_single = self.precision()
        self.recall_single = self.recall()
        self.accuracy_single = self.accuracy()
        self.f_measure_single = self.f_measure()
        if not combinedVote == None:
            self.oneError_single = self.oneError(combinedVote, true_phenotype)
            self.rankLoss_single = self.rankLoss(combinedVote, true_phenotype)

    def getLossSingle(self):
        return self.loss_single

    def getPrecisionSingle(self):
        return self.precision_single

    def getRecallSingle(self):
        return self.recall_single

    def getAccuracySingle(self):
        return self.accuracy_single

    def getFmeasureSingle(self):
        return self.f_measure_single

    def getOneErrorSingle(self):
        return self.oneError_single

    def getRankLossSingle(self):
        return self.rankLoss_single

    def reportMLperformance(self, measureList):
        print("Multi-label performance metrics report.......")
        print("-----------------------------------------------")
        for key, value in measureList.items():
            print(key + " measure value is: " + str(value) + "\n")
        print("-----------------------------------------------")