class Prediction:
    def __init__(self, population):
        self.decision = None

        self.vote = {}
        self.tieBreak_Numerosity = {}
        self.tieBreak_TimeStamp = {}
        self.matchCount = {}

        phenotype_list = []

        for ref in population.matchSet:
            cl = population.popSet[ref]
            if any(cl.phenotype in s for s in population.labelPowerSetList):
                pass
            else:
                population.labelPowerSetList.append(cl.phenotype)

            if any(cl.phenotype in s for s in phenotype_list):
                self.vote[cl.phenotype] += cl.fitness * cl.numerosity
                self.tieBreak_Numerosity[cl.phenotype] += cl.numerosity
                self.tieBreak_TimeStamp[cl.phenotype] += cl.initTimeStamp
                self.matchCount[cl.phenotype] += 1
            else:
                phenotype_list.append(cl.phenotype)
                self.vote[cl.phenotype] = cl.fitness * cl.numerosity
                self.tieBreak_Numerosity[cl.phenotype] = cl.numerosity
                self.tieBreak_TimeStamp[cl.phenotype] = cl.initTimeStamp
                self.matchCount[cl.phenotype] = 1

        self.__nr_of_classes = len(phenotype_list)

        highVal = 0.0
        bestClass = []  # Prediction is set up to handle best class ties for problems with more than 2 classes
        for thisClass in phenotype_list:
            if self.vote[thisClass] >= highVal:
                highVal = self.vote[thisClass]

        for thisClass in phenotype_list:
            if self.vote[thisClass] == highVal:  # Tie for best class
                bestClass.append(thisClass)
        # ---------------------------
        if highVal == 0.0:
            self.decision = None
        # -----------------------------------------------------------------------
        elif len(bestClass) > 1:  # Randomly choose between the best tied classes
            bestNum = 0
            newBestClass = []
            for thisClass in bestClass:
                if self.tieBreak_Numerosity[thisClass] >= bestNum:
                    bestNum = self.tieBreak_Numerosity[thisClass]

            for thisClass in bestClass:
                if self.tieBreak_Numerosity[thisClass] == bestNum:
                    newBestClass.append(thisClass)
            # -----------------------------------------------------------------------
            if len(newBestClass) > 1:  # still a tie
                bestStamp = 0
                newestBestClass = []
                for thisClass in newBestClass:
                    if self.tieBreak_TimeStamp[thisClass] >= bestStamp:
                        bestStamp = self.tieBreak_TimeStamp[thisClass]

                for thisClass in newBestClass:
                    if self.tieBreak_TimeStamp[thisClass] == bestStamp:
                        newestBestClass.append(thisClass)
                # -----------------------------------------------------------------------
                if len(newestBestClass) > 1:  # Prediction is completely tied - eLCS has no useful information for making a prediction
                    self.decision = 'Tie'
                else:
                    self.decision = newestBestClass[0]
            else:
                self.decision = newBestClass[0]
        # ----------------------------------------------------------------------
        else:  # One best class determined by fitness vote
            self.decision = bestClass[0]

    def combinePredictions(self, population):

        self.combPred = None
        self.combVote = []

        for i in range(self.__nr_of_classes):
            self.combVote.append(0.0)

        if len(population.matchSet) == 0:
            self.combPred = None
        else:
            for ref in population.matchSet:
                cl = population.popSet[ref]
                if any(cl.phenotype in s for s in population.labelPowerSetList):
                    pass
                else:
                    population.labelPowerSetList.append(cl.phenotype)

                vote = cl.fitness * cl.numerosity
                it = 0
                for l in cl.phenotype:
                    if (l == '1'):
                        self.combVote[it] += vote
                    it += 1
            maxVote = max(self.combVote)
            if maxVote > 0.0:
                for i in range(len(self.combVote)):
                    self.combVote[i] /= maxVote
            else:
                print('wait')

            pred = []
            loc = 0
            for val in self.combVote:
                if val >= 0.7:
                    pred.append('1')
                else:
                    pred.append('0')
                loc += 1
            self.combPred = "".join(pred)

    def getCombPred(self):
        return self.combPred

    def getCombVote(self):
        return self.combVote

    def getFitnessSum(self, population, low, high):
        """ Get the fitness sum of rules in the rule-set. For continuous phenotype prediction. """
        fitSum = 0
        for ref in population.matchSet:
            cl = population.popSet[ref]
            if cl.phenotype[0] <= low and cl.phenotype[1] >= high:  # if classifier range subsumes segment range.
                fitSum += cl.fitness
        return fitSum

    def getDecision(self):
        return self.decision