from typing import Tuple, Generator

from prolothar_common.models.eventlog import EventLog, Trace, Event
from prolothar_rule_mining.models.event_flow_graph import EventFlowGraph

__EMPTY_EVENT_FLOW_GRAPH = EventFlowGraph()

def eventlog_iterator(eventlog: EventLog) -> Generator[Tuple[Trace, str], None, None]:
    """
    enables iteration on an eventlog that outputs tuples
    (prefix of a trace: Trace, next_activity_name: str)
    """
    source_event = Event(__EMPTY_EVENT_FLOW_GRAPH.source.event)
    for trace in eventlog:
        for i in range(len(trace)):
            yield (
                Trace(
                    '%s-%d' % (str(trace.get_id()), i),
                    [source_event] + trace.events[:i],
                    trace.attributes
                ),
                trace.events[i].activity_name
            )
        yield (
            Trace(
                '%s-%d' % (str(trace.get_id()), len(trace)),
                [source_event] + trace.events,
                trace.attributes
            ),
            __EMPTY_EVENT_FLOW_GRAPH.sink.event
        )

