# -*- coding: utf-8 -*-

#import order is important!
import pathlib
from setuptools import setup
from distutils.extension import Extension
from Cython.Build import cythonize
import os

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

with open(HERE / 'requirements.txt', 'r') as f:
    install_reqs = [
        s for s in [
            line.split('#', 1)[0].strip(' \t\n') for line in f
        ] if '==' in s
    ]

with open(HERE / 'version.txt', 'r') as f:
    version = f.read().strip()

def make_extension_from_pyx(path_to_pyx: str) -> Extension:
    return Extension(
        path_to_pyx.replace('/', '.').replace('.pyx', ''),
        sources=[path_to_pyx], language='c++')

if os.path.exists('prolothar_rule_mining/rule_miner/data_to_sequence/rules/mdl.pyx'):
    extensions = [
        make_extension_from_pyx("prolothar_rule_mining/rule_miner/data_to_sequence/rules/mdl.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/rule_miner/data_to_sequence/rules/rule.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/rule_miner/data_to_sequence/rules/list_of_rules.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/rule_miner/classification/rce/strategy/abstract.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/node.pyx"),
        #alignment
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/alignment_finder.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/a_star.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/path_enumeration.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/partial_alignment.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/alignment.pyx"),
        make_extension_from_pyx("prolothar_rule_mining/models/event_flow_graph/alignment/heuristics.pyx"),
        #conditions
        make_extension_from_pyx("prolothar_rule_mining/models/conditions.pyx"),
    ]
else:
    extensions = []

# This call to setup() does all the work
setup(
    name="prolothar-rule-mining",
    version=version,
    description="algorithms for process mining and data mining on event sequences",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://anonymous/KI/DataScience/processmining/prolothar-process-discovery",
    author="Anonymous",
    author_email="anonymous",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: Process Mining"
    ],
    packages=["prolothar_rule_mining"],
    include_package_data=True,
    ext_modules=cythonize(extensions, language_level = "3"),
    zip_safe=False,
    install_requires=install_reqs
)
