# -*- coding: utf-8 -*-

import os
import pathlib
from setuptools import setup
from distutils.extension import Extension
from Cython.Build import cythonize

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

with open('version.txt', 'r') as f:
    version = f.read().strip()

with open(HERE / 'requirements.txt', 'r') as f:
    install_reqs = [
        s for s in [
            line.split('#', 1)[0].strip(' \t\n') for line in f
        ] if s != ''
    ]

if os.path.exists('dhcollections/list_utils.pyx'):
    extensions = [
        Extension("dhcollections.list_utils",
                sources=["dhcollections/list_utils.pyx"], language='c++'),
    ]
else:
    extensions = []

# This call to setup() does all the work
setup(
    name="dh-custom-collections",
    version=version,
    description="Utils for lists and other collections as well as custom collection classes",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.dillinger.de/KI/DataScience/utils/dh-custom-collections",
    author="Boris Wiegand",
    author_email="boris.wiegand@dillinger.biz",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: Utilities"
    ],
    packages=["dhcollections"],
    ext_modules=cythonize(extensions, language_level = "3"),
    zip_safe=False,
    include_package_data=True,
    install_requires=install_reqs
)