# -*- coding: utf-8 -*-

import unittest
from experimentlogger.experiment_logger import ExperimentLogger
import os
from shutil import rmtree

class TestExperimentLogger(unittest.TestCase):

    def setUp(self):
        self.logger = ExperimentLogger('testtemp', 'Testexperiment')

    def tearDown(self):
        self.logger.close()
        if os.path.exists('testtemp'):
            rmtree('testtemp', ignore_errors=True)

    def test_log(self):
        self.logger.log('Test 1')
        self.logger.log('Test 2')
        self.logger.close()

        with open(os.path.join(
                self.logger.get_path_to_experiment_directory(),
                'log.txt'), "r") as logfile:
            loglines=logfile.readlines()

            self.assertEqual(2, len(loglines))
            self.assertTrue('Test 1' in loglines[0])
            self.assertTrue('Test 2' in loglines[1])

    def test_get_absolute_path(self):
        path = self.logger.get_absolute_path('myfile.png')
        self.assertTrue(os.path.isabs(path))
        self.assertTrue(self.logger.get_path_to_experiment_directory() in path)

    def test_open_text_file(self):
        with self.logger.open_text_file('Textfile') as textfile:
            textfile.write('text')

        with open(os.path.join(
                self.logger.get_path_to_experiment_directory(),
                'Textfile'), "r") as logfile:
            loglines=logfile.readlines()
            self.assertCountEqual(['text'], loglines)

    def test_open_binary_file(self):
        with self.logger.open_binary_file('Binaryfile') as binaryfile:
            binaryfile.write(bytearray(b'{\x03\xff\x00d'))

        with open(os.path.join(
                self.logger.get_path_to_experiment_directory(),
                'Binaryfile'), "rb") as logfile:
            loglines=logfile.readlines()
            self.assertCountEqual([b'{\x03\xff\x00d'], loglines)

if __name__ == '__main__':
    unittest.main()
