# -*- coding: utf-8 -*-

from typing import List

from dhparallel.engine.abstract.computation_engine import ComputationEngine
from dhparallel.engine.multiprocess.partitionable.multiprocess_partitionable_list import MultiprocessPartitionableList

import psutil

class MultiprocessComputationEngine(ComputationEngine):
    """computation engine that distributes computations across the local CPU
    using the multiprocess library"""
    
    def __init__(self, nr_of_workers: int = max(2,psutil.cpu_count())):
        """creates a new MultiprocessComputationEngine
        
        Args:
            nr_of_workers:
                default is max(2, the number of available cores).
                = the number of workers (cores) to use for computations. 
                must be greater 0
        """
        if nr_of_workers <= 0:
            raise ValueError('nr_of_workers must not be <= 0')
        self.__nr_of_workers = nr_of_workers
    
    def create_partitionable_list(self, l: List) -> MultiprocessPartitionableList:
        return MultiprocessPartitionableList(l, self.__nr_of_workers)
    