# -*- coding: utf-8 -*-

import pathlib
import pandas as pd

from prolothar_common.models.eventlog import EventLog
from prolothar_common.models.directly_follows_graph import DirectlyFollowsGraph

HERE = pathlib.Path(__file__).parent

def load_log_for_experiments():
    log = pd.read_csv(
            str(HERE / 'jsbach_chorals_harmony.data'),
            names=['Choral ID', 'EventNumber', 'Pitch_0', 'Pitch_1', 'Pitch_2',
                   'Pitch_3', 'Pitch_4', 'Pitch_5', 'Pitch_6', 'Pitch_7',
                   'Pitch_8', 'Pitch_9', 'Pitch_10', 'Pitch_11', 'Bass', 'Meter',
                   'Chord'],
            sep=',',)

    log = EventLog.create_from_pandas_df(
            log, 'Choral ID', 'Chord',
            event_attribute_columns=['EventNumber', 'Pitch_0', 'Pitch_1', 'Pitch_2',
                   'Pitch_3', 'Pitch_4', 'Pitch_5', 'Pitch_6', 'Pitch_7',
                   'Pitch_8', 'Pitch_9', 'Pitch_10', 'Pitch_11', 'Bass', 'Meter'],
            trace_attribute_columns=[])

    #there must be a start and an end event
    log.add_start_activity_to_every_trace('START')
    log.add_end_activity_to_every_trace('END')

    return log

if __name__ == '__main__':
    log = load_log_for_experiments()
    print(log.get_nr_of_traces())
    DirectlyFollowsGraph.create_from_event_log(log).plot()
