import os
import pathlib
import json

from prolothar_common.models.eventlog import EventLog

from prolothar_common.models.dataset import TargetSequenceDataset
from prolothar_common.models.dataset.instance import TargetSequenceInstance

HERE = pathlib.Path(__file__).parent

def load_class_level_eventlog() -> EventLog:
    with open(os.path.join(HERE, 'class_level_log.json')) as f:
        return EventLog.create_from_dict(json.load(f))

def load_method_level_eventlog() -> EventLog:
    with open(os.path.join(HERE, 'method_level_log.json')) as f:
        return EventLog.create_from_dict(json.load(f))

def _event_log_to_dataset(eventlog: EventLog) -> TargetSequenceDataset:
    dataset = TargetSequenceDataset(
        ['v', 'h', 'dryRun', 'skipUpscaling', 'clean', 'androidMipmapInsteadOfDrawable',
         'androidIncludeLdpiTvdpi', 'antiAliasing', 'roundingMode', 'outCompression'],
        ['scale', 'compressionQuality']
    )
    for trace in eventlog:
        dataset.add_instance(TargetSequenceInstance(
            trace.get_id(), trace.attributes, trace.to_activity_list()
        ))
    return dataset

def load_class_level_dataset() -> TargetSequenceDataset:
    return _event_log_to_dataset(load_class_level_eventlog())

def load_method_level_dataset() -> TargetSequenceDataset:
    return _event_log_to_dataset(load_method_level_eventlog())