# -*- coding: utf-8 -*-

from typing import List

from random import Random
import pathlib

from prolothar_common.models.eventlog import EventLog, Trace, Event

HERE = pathlib.Path(__file__).parent

CATEGORIES = [
        'frontpage',
        'news',
        'tech',
        'local',
        'opinion',
        'on-air',
        'misc',
        'weather',
        'msn-news',
        'health',
        'living',
        'business',
        'msn-sports',
        'sports',
        'summary',
        'bbs',
        'travel'
    ]

def load_log_for_experiments(max_nr_of_traces: int = None,
                             random: Random = None):
    log = EventLog()
    with open(HERE / 'msnbc990928.seq', 'r') as f:
        for i,line in enumerate(_read_lines(f, max_nr_of_traces, random)):
            log.add_trace(_parse_line_to_trace(i, line))

    #there must be a start and an end event
    log.add_start_activity_to_every_trace('START')
    log.add_end_activity_to_every_trace('END')

    return log

def _parse_line_to_trace(i: int, line: str) -> Trace:
    return Trace(i, [Event(CATEGORIES[int(category_nr) - 1])
                     for category_nr in line.split()])


def _read_lines(f, max_nr_of_traces: int, random: Random) -> List[str]:
    if random is None:
        random = Random()
    lines = f.readlines()
    if max_nr_of_traces is not None and max_nr_of_traces < len(lines):
        random.shuffle(lines)
        return lines[:max_nr_of_traces]
    return lines
