# -*- coding: utf-8 -*-
import os
import pathlib
import pickle

from pm4py.objects.log.importer.xes import importer as xes_import_factory

import prolothar_common.pm4py_utils as pm4py_utils

from prolothar_common.models.eventlog import Event
from prolothar_common.models.dataset import TargetSequenceDataset
from prolothar_common.models.dataset.instance import TargetSequenceInstance

HERE = pathlib.Path(__file__).parent

def read_log():
    path_to_eventlog = os.path.join(HERE, 'traffic_fines_log.pickle')

    if os.path.exists(path_to_eventlog):
        with open(path_to_eventlog, 'rb') as f:
            log = pickle.load(f)
    else:
        xes = xes_import_factory.apply(str(HERE / 'Road_Traffic_Fine_Management_Process.xes.gz'))

        log = pm4py_utils.convert_pm4py_log(xes)

        try:
            with open(path_to_eventlog, 'wb') as f:
                pickle.dump(log, f)
        except PermissionError as e:
            print('No permission to write TrafficFines event log to disk for caching:')
            print(e)

    return log

def load_dataset() -> TargetSequenceDataset:
    path_to_dataset = os.path.join(HERE, 'traffic_fines_dataset.pickle')

    if os.path.exists(path_to_dataset):
        with open(path_to_dataset, 'rb') as f:
            dataset = pickle.load(f)
    else:
        dataset = TargetSequenceDataset(
            ['org:resource', 'dismissal', 'vehicleClass', 'article'],
            ['amount', 'totalPaymentAmount', 'points']
        )

        for trace in read_log():
            instance_attributes = {}
            for attribute in dataset.get_attributes():
                instance_attributes[attribute.get_name()] = trace.events[0].attributes[attribute.get_name()]
            dataset.add_instance(TargetSequenceInstance(
                trace.get_id(), instance_attributes, trace.to_activity_list()
            ))

        try:
            with open(path_to_dataset, 'wb') as f:
                pickle.dump(dataset, f)
        except PermissionError as e:
            print('No permission to write TrafficFines dataset to disk for caching:')
            print(e)

    return dataset
