#include <iostream>

#include "pattern.h"
#include "patternset.h"
#include "database.h"

int main(int argc, char *argv[])
{

    const std::string data_file(argv[1]);
    const std::string output_file(argv[2]);
    Database db (data_file, FICSIM);
    auto dim = db.getDimensions();
    db.transposeToColumn();
    Pattern::nrow = dim.first;
    Pattern::ncol = dim.second;
    Pattern::fullData = std::make_shared<dbitvec>(Pattern::nrow);
    Pattern::fullData->setOnes(Pattern::nrow);

    PatternSet ps (db.begin(), db.end(), db.getIDMap());

    std::list<Pattern::Candidate> cands;
    // while we can generate candidates with gain
    uint32_t i = 1;
    while ((cands = ps.genCandidates()).size() > 0)
    {
        std::cout << "Iteration " << i << "\n";
        std::cout << "Candidate: \n\t";
        cands.front().newPat->print(std::cout, db.getIDMap());
        // cands.front().newPat->print(std::cout);
        std::cout << "\nMerged from " << cands.front().type << ":\n\t";
        cands.front().pat1->print(std::cout, db.getIDMap());
        // cands.front().pat1->print(std::cout);
        std::cout << "\n\t";
        cands.front().pat2->print(std::cout, db.getIDMap());
        // cands.front().pat2->print(std::cout);
        std::cout << "\nGain: " << cands.front().gain;
        std::cout << "\n\n";

        // TODO
        // TODO
        // TODO: only test for best candidate right now
        ps.commitCandidate(cands.front());

        std::cout << "\nCurrent CL: " << ps.computeCL() << "\n\n";
        ++i;
    }

    std::ofstream ofs(output_file);

    ps.printToFile(ofs);

    ofs.close();

    return 0;
}
