#include <iostream>
#include <algorithm>
#include <vector>

#include "../misc/EncodingCalucator.h"
#include <unordered_map>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <queue>
#include <unordered_set>
#include <cstdint>

#include "../misc/Data.h"
#include "../misc/DataCover.h"
#include "../misc/Mwindow.h"
#include "../misc/common_functions.h"

using namespace std;

typedef int32_t int32;
typedef uint32_t uint32;

class PatternComparator
{
public:
	vector<double> values_to_be_compared;
	int order_step;

	PatternComparator(vector<Pattern*> &list_of_patterns , int order_step)
	{
		//cout << "Num of patterns: " << list_of_patterns.size() << endl;
		switch (order_step)
		{
		case 1: init_lex( list_of_patterns ); break;
		case 2:	init_lst( list_of_patterns ); break;
		case 3:	init_supp( list_of_patterns ); break;
		case 4: init_estgain( list_of_patterns ); break;
		case 5:	init_patlen( list_of_patterns ); break;
		case 6: ; break;
		}

		/*cout << "val comp array: " << endl;
		for(int  i = 0; i < values_to_be_compared.size(); i++)
		{
			cout << values_to_be_compared[i] << endl;
		}
		cout << " ---------------------------- " << endl;*/

		this->order_step = order_step;
	}

	bool operator() (uint32 i, uint32 j)
	{
		switch (order_step)
		{
		case 1: return less_lex( i, j); break;
		case 2:	return less_lst( i, j); break;
		case 3:	return less_supp( i, j); break;
		case 4:	return less_estgain( i, j); break;
		case 5:	return less_patlen( i, j); break;
		case 6: ; break;
		}
	}

	void init_estgain( vector<Pattern*> &list_of_patterns )
	{
		for(uint32  i = 0; i < list_of_patterns.size(); i++ )
		{
			values_to_be_compared.push_back( (double)list_of_patterns[i]->getPatternNum() );
		}
	}

	void init_patlen( vector<Pattern*> &list_of_patterns )
	{
		for(uint32  i = 0; i < list_of_patterns.size(); i++ )
		{
			values_to_be_compared.push_back( (double)list_of_patterns[i]->getSize() );
		}
	}

	void init_supp( vector<Pattern*> &list_of_patterns )
	{
		for(uint32  i = 0; i < list_of_patterns.size(); i++ )
		{
			values_to_be_compared.push_back( (double)list_of_patterns[i]->getUsage() );
		}

	}

	void init_lst( vector<Pattern*> &list_of_patterns )
	{
		for(uint32  i = 0; i < list_of_patterns.size(); i++ )
		{
			values_to_be_compared.push_back((double)list_of_patterns[i]->getPatternStLength() );
		}

	}

	void init_lex( vector<Pattern*> &list_of_patterns )
	{
		for(uint32  i = 0; i < list_of_patterns.size(); i++ )
		{
			int lex_val = 0;
			for(uint32 j = 0; j < list_of_patterns[i]->getSize(); j++)
			{
				lex_val += (int)list_of_patterns[i]->charAt(j);
			}
			values_to_be_compared.push_back( lex_val );
		}

	}

	bool less_patlen(uint32 i, uint32 j)
	{
		return values_to_be_compared[i] > values_to_be_compared[j];
	}

	bool less_supp(uint32 i, uint32 j)
	{
		//cout << i << " - and - "<< j << endl;
		return values_to_be_compared[i] > values_to_be_compared[j];
	}

	bool less_lst(uint32 i, uint32 j)
	{
		return ( values_to_be_compared[i] < values_to_be_compared[j] );
	}

	bool less_lex(uint32 i, uint32 j)
	{
		return ( values_to_be_compared[i] < values_to_be_compared[j] );
	}

	bool less_estgain(uint32 i, uint32 j)
	{
		return ( values_to_be_compared[i] < values_to_be_compared[j] );
	}
};


int calc_supp(Data *D, Pattern *X) {

	int supp = 0;
	for(int i = 0; i<D->getSize(); i++)
	{
		if( D->charAt(i) == X->charAt(0) )
		{
			int k = 1, j = i + 1;
			while( j < D->getSize() )
			{
				if( D->charAt(j) == X->charAt(k) )
				{
					k++;
					if( k == X->getSize() )
					{
						supp ++;
						break;
					}
				}
				j++;
			}
		}
	}
	return supp;
}


void init_pattern_supp( Data *D, vector<Pattern*> &list_of_patterns )
{
	for(uint32 i = 0; i < list_of_patterns.size() ; i++)
	{
		list_of_patterns[i]->setUsage( calc_supp(D, list_of_patterns[i]) );
	}
}

void init_pattern_st_len( Data *D, vector<Pattern*> &list_of_patterns )
{
	for(uint32 i = 0; i < list_of_patterns.size() ; i++)
	{
		double lst = 0;
		for(int j = 0; j < list_of_patterns[i]->getSize(); j++)
		{
			lst += log2( (double) D->getSize() / D->getAlphabetSupport( list_of_patterns[i]->charAt(j) ) );
		}
		list_of_patterns[i]->setPatternStLength(lst);
	}
}

void init_sorted_pattern_list( vector<vector<Mwindow*>> &list_of_mwindows,
		vector<vector<Mwindow*>> &sorted_list_of_mwindows, vector<int> &pattern_index )
{
	sorted_list_of_mwindows.clear();
	for(uint32 i = 0; i < list_of_mwindows.size() ; i++)
	{
		sorted_list_of_mwindows.push_back( list_of_mwindows[pattern_index[i]] );
		sorted_list_of_mwindows[i][0]->pattern->setSortIndex( i );
		//sorted_list_of_mwindows[i][0]->pattern->printPattern();
		//cout << "Sort Index: " << sorted_list_of_mwindows[i][0]->pattern->getSortIndex() << endl;
	}
}

/*void reset_sort_index( vector<Pattern*> &list_of_patterns )
{
	for(int i = 0; i < list_of_patterns.size(); i++ )
	{
		list_of_patterns[i]->setSortIndex( 0 );
	}
}*/

void sort_patterns_in_candidate_order(vector<int> &pattern_index, vector<Pattern*> &list_of_patterns )
{
	//cout << "Trying to sort" << endl;
	pattern_index.clear();
	for(uint32 i = 0; i < list_of_patterns.size() ; i++)
	{
		pattern_index.push_back( i );
	}

	for(int i = 1; i < 6; i++)
	{
		stable_sort( pattern_index.begin() , pattern_index.end() , PatternComparator( list_of_patterns , i )  );
	}
	//cout << "Done sorting" << endl;
}



/*
Mwindow* find_window( Pattern *pat, Data *D, unordered_set &cover, int start_pos, int end_pos )
{
	for(int data_idx = start_pos; data_idx <= end_pos; data_idx++ )
	{
		int pat_idx = 0;
		if( D->charAt( data_idx ) == pat->charAt( 0 ) )
		{
			pat_idx ++;
			for( int data_s_idx = data_idx + 1; data_s_idx <= end_pos; data_s_idx ++ )
			{
				if( data_s_idx == pat->charAt( pat_idx ) )
				{
					pat_idx++;
				}
				if( pat_idx == pat->getSize() )
				{

				}
			}
		}
	}
}*/

void cover_all_pos_in_win( bool *covered_pos, Mwindow* win )
{
	for( uint32 i = 0; i < win->getUsedLen(); i++ )
	{
		covered_pos[ win->getPositionInData(i) ] = true;
	}
}

int find_all_windows_in_search_extent( vector<Mwindow*> &all_windows_in_search_extent, vector<Mwindow*> &wins_of_pattern_i,
		uint32 start_pos, uint32 end_pos, uint32 last_visited_index, int32 &search_extend_end_index, bool &fine_search )
{
	uint32 win_idx = last_visited_index;
	fine_search = false;
	if( win_idx < wins_of_pattern_i.size() )
	{
		search_extend_end_index = wins_of_pattern_i[ win_idx ]->getStartIndex() - 1; // BUG!!
	}
	while( win_idx < wins_of_pattern_i.size() && wins_of_pattern_i[ win_idx ]->getStartIndex() <= end_pos )
	{
		if( wins_of_pattern_i[ win_idx ]->getStartIndex() >= start_pos && wins_of_pattern_i[ win_idx ]->getEndIndex() <= end_pos )
		{
			all_windows_in_search_extent.push_back( wins_of_pattern_i[ win_idx ] );
			if( (int32)wins_of_pattern_i[ win_idx ]->getStartIndex() <= search_extend_end_index )
			{
				fine_search = true;
			}
			if( (int32)wins_of_pattern_i[ win_idx ]->getEndIndex() > search_extend_end_index )
			{
				search_extend_end_index = wins_of_pattern_i[ win_idx ]->getEndIndex();
			}
		}
		win_idx ++;
	}
	return win_idx - 1;
}

void add_windows_to_choosen_list( vector<Mwindow*> &tmp_choosen_win, vector<Mwindow*> &choosen_windows, bool *covered_pos )
{
	choosen_windows.clear();
	for(uint32 i = 0; i < tmp_choosen_win.size(); i++ )
	{
		choosen_windows.push_back( tmp_choosen_win[i] );
		cover_all_pos_in_win( covered_pos, tmp_choosen_win[i] );
	}
}

void find_interleaved_wins( Data *D, Pattern *p, uint32 start_pos, uint32 end_pos, bool *covered_pos, vector<Mwindow*> &interleaved_win )
{
	for(uint32 i = start_pos; i <= end_pos; i++)
	{
		if( !covered_pos[i] )
		{
			if( D->charAt(i) == p->charAt(0) )
			{

				Mwindow *tmw;
				vector<uint32> pos;
				pos.push_back( i );
				uint32 j = i + 1, k = 1;
				bool flag = true;
				uint32 gapcount = 0;
				while(flag && j < D->getSize())
				{
					//cout << j << " " << k << " " << x[k] << endl;
					if( p->charAt(k) == D->charAt(j) && !covered_pos[j])
					{
						pos.push_back( j );
						k++;
					}
					else if( D->charAt(j) == -1 )
					{
						flag = true;
						break;
					}
					else
					{
						gapcount++;
					}
					if(k == p->getSize())
					{
						flag = false;
						tmw = new Mwindow(i, j, p, 0, pos, gapcount, p->getSize());
						i = j;
					}
					j++;
				}
				if(!flag && tmw->getNumOfGaps() <= p->getSize())
				{

					interleaved_win.push_back( tmw );
				}

				pos.clear();
			}
		}
	}
}

void insert_win_in_sorted_order_at_back( vector<Mwindow*> &a, vector<Mwindow*> &b )
{
	for(uint32 i = 0; i < b.size(); i++ )
	{
		int j = a.size() - 1;
		while( a[j]->getStartIndex() > b[i]->getStartIndex() )
		{
			j--;
		}
		a.insert( a.begin() + j + 1, b[i]);
	}
}
/*
int index_of_next_min_win( int last_visited_index, int end_index_of_last_choosen_win, vector<Mwindow*> list_of_win_of_pattern_i )
{
	for( int i = last_visited_index; i < list_of_win_of_pattern_i.size(); i++ )
	{
		if( list_of_win_of_pattern_i[ i ]->getStartIndex() > end_index_of_last_choosen_win )
		{
			return list_of_win_of_pattern_i[ i ]->getStartIndex();
		}
	}
}*/

bool continue_interleaving( Mwindow* new_choosen_win, int32 end_of_old_window_extend )
{
	return (int32)new_choosen_win->getStartIndex() <= end_of_old_window_extend ;
}


int32 get_new_end_of_window_extend( Mwindow* new_choosen_win, int32 end_of_old_window_extend )
{
	return max( end_of_old_window_extend, (int32)new_choosen_win->getEndIndex() );
}

bool interleaving_possible( vector<Mwindow*> &wins_of_pattern_i, uint32 last_visited_index, uint32 interleaving_window_extend_start, uint32 interleaving_window_extend_end )
{
	bool flag = false;
	for( uint32 i = last_visited_index; i < wins_of_pattern_i.size(); i++ )
	{
		Mwindow *next_window = wins_of_pattern_i[ i ];
		if( next_window->getStartIndex() >= interleaving_window_extend_start && next_window->getEndIndex() <= interleaving_window_extend_end )
		{
			flag = true;
			break;
		}
		if( next_window->getStartIndex() >= interleaving_window_extend_end )
		{
			break;
		}
	}

	return flag;
}

class WindowComparator
{
public:
	vector<uint32> gaps;

	WindowComparator( vector<Mwindow*> &all_win_in_search_extent )
	{
		for(uint32 i = 0; i < all_win_in_search_extent.size(); i++ )
		{
			gaps.push_back( (uint32)all_win_in_search_extent[i]->getNumOfGaps() );
		}
	}

	bool operator() (uint32 i, uint32 j)
	{
		return gaps[i] < gaps[j];
	}
};

void sort_windows( vector<uint32> &indexes, vector<Mwindow*> &all_win_in_search_extent )
{
	stable_sort( indexes.begin() , indexes.end() , WindowComparator( all_win_in_search_extent )  );
}

void choose_windows_to_add_by_gap( vector<uint32> &sorted_indexes, vector<Mwindow*> &all_win_in_search_extent, vector<bool> &choosen_indexes, uint32 start_search_extend, uint32 length_of_search_extend )
{
	vector<bool> covered_pos ( length_of_search_extend, false );
	for( uint32 i = 0; i < sorted_indexes.size(); i++ )
	{
		bool flag = true;
		for( uint32 j = 0; j < all_win_in_search_extent[sorted_indexes[i]]->getUsedLen(); j++ )
		{
			uint32 shifted_pos = all_win_in_search_extent[sorted_indexes[i]]->getPositionInData( j ) - start_search_extend;
			if( shifted_pos >= covered_pos.size() )
			{

				cout << "Error!" << j << " - "<< all_win_in_search_extent[sorted_indexes[i]]->getPositionInData( j ) << " - " << start_search_extend << endl;
				exit(0);
			}
			if( covered_pos[ shifted_pos ] )
			{
				flag = false;
			}
		}
		if( flag )
		{
			choosen_indexes[ sorted_indexes[i] ] = true;
			for( uint32 j = 0; j < all_win_in_search_extent[sorted_indexes[i]]->getUsedLen(); j++ )
			{
				uint32 shifted_pos = all_win_in_search_extent[sorted_indexes[i]]->getPositionInData( j ) - start_search_extend;
				covered_pos[ shifted_pos ] = true;
			}
		}
	}
}

void add_windows_to_temp_list( vector<Mwindow*> &tmp_choosen_win, vector<Mwindow*> &all_win_in_search_extent, vector<bool> &choosen_indexes )
{
	for(uint32 i = 0; i < all_win_in_search_extent.size(); i++ )
	{
		if( choosen_indexes[i] )
		{
			tmp_choosen_win.push_back( all_win_in_search_extent[i] );
		}
	}
}

void choose_windows( uint32 pattern_idx, vector<vector<Mwindow*>> &list_of_mwindows,
		vector<Mwindow*> &choosen_windows, Data *D, bool *covered_pos, int start_pos, int end_pos )
{
	if( pattern_idx < list_of_mwindows.size() )
	{
		//cout << "iter: ------------------------------------------------------" << endl;
		uint32 last_visited_idx = -1;
		vector<Mwindow*> tmp_choosen_win, tmp_interleaved_win;
		uint32 start_of_interleaving_search_area = 0;
		int32 end_of_old_window_extend = start_pos - 1;
		int32 found_window_end_index = 0;
		bool fine_search = false;
		vector<Mwindow*> all_win_in_search_extent;
		for( uint32 i = 0; i <= choosen_windows.size(); i++ )
		{
			/*if( i < choosen_windows.size() )
			{
				cout << "CI: "<< continue_interleaving( choosen_windows[i], end_of_old_window_extend ) << endl;
			}*/
			if( i == 0 || i == choosen_windows.size() || !continue_interleaving( choosen_windows[i], end_of_old_window_extend ) )
			{

				//
				if( i > 0 && i < choosen_windows.size() && interleaving_possible( list_of_mwindows[ pattern_idx ], last_visited_idx + 1, start_of_interleaving_search_area + 1, end_of_old_window_extend ) )
				{
					tmp_interleaved_win.clear();
					find_interleaved_wins( D, list_of_mwindows[ pattern_idx ][0]->pattern, start_of_interleaving_search_area + 1 , end_of_old_window_extend, covered_pos, tmp_interleaved_win );
					if( tmp_interleaved_win.size() > 0 )
					{
						//cout << tmp_interleaved_win.size() << " - Interleaved wins found!" << endl;
						/*for( uint32 j = 0; j < tmp_interleaved_win.size(); j++ )
						{
							cout << "Pattern: -- " ;
							tmp_interleaved_win[j]->pattern->printPattern();
							cout << "Position: " <<  tmp_interleaved_win[j]->getStartIndex() << " -- " << tmp_interleaved_win[j]->getEndIndex() << endl;
							for( int k = tmp_interleaved_win[j]->getStartIndex() - 2; k < tmp_interleaved_win[j]->getEndIndex() + 2; k++ )
							{
								cout << D->charAt( k ) << " , ";
							}
							cout << endl;*/


					}
					insert_win_in_sorted_order_at_back( tmp_choosen_win, tmp_interleaved_win );
				}

				uint32 old_tmp_win_size = tmp_choosen_win.size();
				all_win_in_search_extent.clear();
				found_window_end_index = 0;
				if( i < choosen_windows.size() )
				{
					last_visited_idx = find_all_windows_in_search_extent( all_win_in_search_extent, list_of_mwindows[ pattern_idx ], end_of_old_window_extend + 1, choosen_windows[i]->getStartIndex() - 1, last_visited_idx + 1, found_window_end_index, fine_search );
					//cout << end_of_old_window_extend + 1 << " -- " << choosen_windows[i]->getStartIndex() - 1 << endl;
				}
				else
				{
					last_visited_idx = find_all_windows_in_search_extent( all_win_in_search_extent, list_of_mwindows[ pattern_idx ], end_of_old_window_extend + 1, end_pos, last_visited_idx + 1, found_window_end_index, fine_search );
					//cout << end_of_old_window_extend + 1 << " -- " << end_pos << endl;
				}
				if( all_win_in_search_extent.size() > 0 )
				{
					if( fine_search )
					{

						uint32 found_window_start_index = all_win_in_search_extent[0]->getStartIndex();
						uint32 found_window_extend = found_window_end_index - found_window_start_index + 1;
						//cout << pattern_idx << " - Fine tuning: area: " << found_window_extend << "  - begin: " << found_window_start_index << " - end: " << found_window_end_index << " - num of win: " << all_win_in_search_extent.size() << endl;
						vector<uint32> indexes;
						vector<bool> choosen_indexes;
						for(uint32 j = 0; j < all_win_in_search_extent.size(); j++ )
						{
							indexes.push_back( j );
							choosen_indexes.push_back( false );
						}
						sort_windows( indexes, all_win_in_search_extent );
						choose_windows_to_add_by_gap( indexes, all_win_in_search_extent, choosen_indexes, found_window_start_index, found_window_extend );
						int32 old_num_of_win = tmp_choosen_win.size();
						add_windows_to_temp_list( tmp_choosen_win, all_win_in_search_extent, choosen_indexes );
						//cout << " Added: " << tmp_choosen_win.size() - old_num_of_win << endl;
					}
					else
					{
						vector<bool> choosen_indexes;
						for(uint32 j = 0; j < all_win_in_search_extent.size(); j++ )
						{
							choosen_indexes.push_back( true );
						}
						add_windows_to_temp_list( tmp_choosen_win, all_win_in_search_extent, choosen_indexes );
					}

				}
				if( i < choosen_windows.size() )
				{
					if( old_tmp_win_size < tmp_choosen_win.size() )
					{
						start_of_interleaving_search_area = tmp_choosen_win.back()->getEndIndex() + 1;
					}
					else
					{
						start_of_interleaving_search_area = choosen_windows[i]->getStartIndex();
					}
				}
			}
			if( i < choosen_windows.size() )
			{
				tmp_choosen_win.push_back( choosen_windows[i] );
			//cover_all_pos_in_win( covered_pos, choosen_windows[i] );
				end_of_old_window_extend = get_new_end_of_window_extend( choosen_windows[i], end_of_old_window_extend );
			}
		}
		add_windows_to_choosen_list( tmp_choosen_win, choosen_windows, covered_pos );
		choose_windows( pattern_idx + 1, list_of_mwindows, choosen_windows, D, covered_pos, start_pos, end_pos );
	}
}

void reuse_previous_cover_and_choose_windows(vector<Mwindow*> &selected_windows, vector<vector<Mwindow*>> &sorted_list_of_mwindows, Pattern *new_candidate, Data *D, int end_point)
{
	vector<Mwindow*> filtered_selected_windows;
	for(uint32  i = 0; i < selected_windows.size(); i++)
	{

		if( selected_windows[i]->pattern->getSortIndex() < new_candidate->getSortIndex() )
		{
			filtered_selected_windows.push_back( selected_windows[i] );
		}
	}
	selected_windows.clear();
	bool covered_pos[ end_point ] = { false };
	for(uint32  i = 0; i < filtered_selected_windows.size(); i++)
	{
		selected_windows.push_back( filtered_selected_windows[i] );
		cover_all_pos_in_win( covered_pos, filtered_selected_windows[i] );
	}
	//cout << "Filtered win size: " << selected_windows.size() << endl;
	choose_windows( new_candidate->getSortIndex(), sorted_list_of_mwindows, selected_windows, D, covered_pos, 0, end_point );
}

void reset_pattern_usages_gaps_and_stops( vector<Pattern*> &list_of_patterns )
{
	for(uint32  i = 0; i < list_of_patterns.size(); i++ )
	{
		list_of_patterns[i]->setUsage( 0 );
		list_of_patterns[i]->setGaps( 0 );
		list_of_patterns[i]->setStops( 0 );
	}
}

double calc_code_len( vector<Mwindow*> &choosen_windows, vector<Pattern*> &list_of_patterns, Data *D, vector<vector<Mwindow*>> &list_of_mwindows, vector<int32> &susages,
		vector<ChoicePattern> &list_of_choice_patterns, bool prune_mode )
{
	susages.clear();
	for(uint32  i = 0; i <= D->getMaxAlphaNum(); i++ )
	{
		susages.push_back( D->getAlphabetSupport( i ) );
	}
	//cout << "Choosen wins: " << choosen_windows.size() << endl;
	reset_pattern_usages_gaps_and_stops( list_of_patterns );
	//cout << "code len calc 1" << endl;
	for( uint32  i = 0; i < choosen_windows.size(); i++ )
	{
		Pattern* pattern_in_win = choosen_windows[i]->pattern;
		int32 curr_usage = pattern_in_win->getUsage();
		int32 curr_gaps = pattern_in_win->getGaps();
		pattern_in_win->setUsage( curr_usage + 1 );
		pattern_in_win->setGaps( curr_gaps + choosen_windows[i]->getNumOfGaps() );
		if( choosen_windows[i]->getUsedLen() != pattern_in_win->getSize() )
		{
			pattern_in_win->setStops( pattern_in_win->getStops() + 1 );
		}
		for(uint32 j = 0; j < choosen_windows[i]->getUsedLen() ; j++ )
		{
			susages[ pattern_in_win->charAt(j) ] -= 1 ;
		}
		//cout << "Stops: " << pattern_in_win->getStops() << endl;
	}


	int pattern_stream_length = 0, sum_of_pattern_usages = 0 , num_of_patterns = 0, num_of_used_chars = 0;

	for(uint32 i = 0; i < list_of_patterns.size(); i++)
	{
		sum_of_pattern_usages += list_of_patterns[i]->getUsage();

		if( list_of_patterns[i]->getUsage() > 0 )
		{
			num_of_patterns++;
		}
	}

	for(uint32 i = 0; i < susages.size(); i++)
	{
		if( susages[i] > 0 )
		{
			num_of_used_chars++;
		}
	}

	pattern_stream_length = sum_of_pattern_usages;

	EncodingCalucator ec;

	double lctc = ec.ln( D->getMaxAlphaNum() + 1 ) + ec.approxlogchoose(D->getSize() - 1, D->getMaxAlphaNum() ) + ec.ln( num_of_patterns + 1 ) ;

	lctc += ec.ln( sum_of_pattern_usages + 1 ) + ec.approxlogchoose( sum_of_pattern_usages - 1, num_of_patterns - 1) ;

	for(unsigned int i = 0; i < list_of_choice_patterns.size(); i++)
	{

		lctc += list_of_choice_patterns[i].get_codelen( D );

	}
	double prev_lctc = lctc;
	//cout << "Prune mode :" << prune_mode << endl;
	if( !prune_mode )
	{
		int choice_pattern_that_can_be_added = check_if_new_pattern_can_be_made_a_choice_pattern( list_of_patterns[ list_of_patterns.size() - 1 ], list_of_choice_patterns, D );
		if( choice_pattern_that_can_be_added != -1 )
		{
			//cout << "Added as choice pattern " << endl;
			lctc += -list_of_choice_patterns[ choice_pattern_that_can_be_added ].change_in_codelen_if_added( list_of_patterns[ list_of_patterns.size() - 1 ], D );
			//list_of_choice_patterns[ choice_pattern_that_can_be_added ].print();
		}
		else
		{
			//cout << "Added as simple pattern " << endl;
			ChoicePattern simple_cp( list_of_patterns[ list_of_patterns.size() - 1 ], D );
			lctc += simple_cp.get_codelen( D );
		}
		//cout << "Added to lctc: " << (lctc - prev_lctc) << endl;
	}
	//cout << "sum" << sum_of_usages << endl;
	/*for(uint32 i = 0; i < list_of_patterns.size(); i++)
	{

		lctc += ec.ln( list_of_patterns[i]->getSize() );
		if( !prequential )
		{
			lctc += ec.ln( list_of_patterns[i]->getGaps() + 1 );
		}
		//cout << "Gaps: " << i << " " <<  XX[i]->getGaps() << endl;
		for(int j = 0; j < list_of_patterns[i]->getSize(); j++ )
		{
			lctc += ec.lu( (double)D->getSize() / D->getAlphabetSupport(list_of_patterns[i]->charAt(j)) );
		}

	}*/
	//cout << "lctd: " << lctd << endl;
	double ldct = ec.ln( D->getSize() ) + ec.ln( D->getNumOfSeq() );

	for(uint32 i = 0; i < susages.size(); i++ )
	{
		pattern_stream_length += susages[i];
	}

	for(uint32 i = 0; i < susages.size(); i++ )
	{
		if( susages[i] > 0 )
		{
			ldct += susages[i] * ec.lu((double)pattern_stream_length / susages[i]);
		}
		else if( susages[i] < 0 )
		{
			cout << "usage less than zero for: " << "i: " << i << " ->> " << susages[i] << endl;
			//exit(0);
		}
	}

	//cout << "ldct: " << ldct << endl;

	for(uint32 i = 0; i < list_of_patterns.size(); i++)
	{
		if( list_of_patterns[i]->getUsage() > 0 )
		{
			ldct += list_of_patterns[i]->getUsage() * ec.lu((double)pattern_stream_length / list_of_patterns[i]->getUsage());
			int fills = list_of_patterns[i]->getUsage() * ( list_of_patterns[i]->getSize() -1 );
			int gaps = list_of_patterns[i]->getGaps();
			double pcl = exact_prequential_length_of_binary_sequence( fills, gaps, 0.5 );
			ldct += pcl;
		}
	}
	/*cout << "Sum of usages: " << sum_of_usages << endl;
	cout << "Length of pattern stream: " << pattern_stream_length << endl;
	cout << "Calculated code length ldct: " << (ldct) << endl;
	cout << "Calculated code length lctc: " << (lctc) << endl;*/
	return ldct + lctc;

}

void check_pattern_nums( vector<Pattern*> &list_of_patterns )
{
	for(uint32 i = 0; i < list_of_patterns.size(); i++ )
	{
		for(uint32 j = 0; j < list_of_patterns.size(); j++ )
		{
			if( i != j && list_of_patterns[i]->getPatternNum() == list_of_patterns[j]->getPatternNum() )
			{
				cout << "Same pattern num" << endl;
				exit(0);
			}
		}
	}
}

void print_patterns_and_mwindows( vector<vector<Mwindow*>> &sorted_list_of_mwindows )
{
	for(uint32  i = 0; i < sorted_list_of_mwindows.size(); i++)
	{
		sorted_list_of_mwindows[i][0]->pattern->printPattern();
		cout << "Num of mwindows: " << sorted_list_of_mwindows[i].size() << " - Sort pos: " << sorted_list_of_mwindows[i][0]->pattern->getSortIndex() << endl;
	}
}

void check_patterns_sync( vector<vector<Mwindow*>> &list_of_mwindows, vector<Pattern*> &list_of_patterns )
{
	for(uint32 i = 0; i < list_of_mwindows.size(); i++)
	{

		if( list_of_mwindows[i][0]->pattern->getPatternNum() != list_of_patterns[i]->getPatternNum() )
		{
			cout << "Lists not in sync" << endl;
			exit(0);
		}

	}
}

int greedy_ditto( Data *D, vector<Pattern*> &list_of_patterns , double &new_code_len, vector<vector<Mwindow*>> &list_of_mwindows,
		vector<Mwindow*> &selected_windows, vector<int32> &new_susages, Pattern *new_candidate, vector<ChoicePattern> &list_of_choice_patterns, bool prune_mode)
{
	//check_pattern_nums( list_of_patterns );
	//check_patterns_sync( list_of_mwindows, list_of_patterns );
	//cout << " In greedy ditto " << endl;
	//init_pattern_supp( D, list_of_patterns );
	//cout << "mwindow list len: " << list_of_mwindows.size() << endl;
	//cout << "pattern list len: " << list_of_patterns.size() << endl;
	if( list_of_mwindows.size() != list_of_patterns.size() )
	{
		cout << "Lengths not equal" << endl;
		exit(0);
	}
	init_pattern_st_len( D, list_of_patterns );

	vector<int32> pattern_index;

	sort_patterns_in_candidate_order( pattern_index, list_of_patterns );

	vector<vector<Mwindow*>> sorted_list_of_mwindows;
	//cout << "before sorting windows" << endl;
	init_sorted_pattern_list( list_of_mwindows , sorted_list_of_mwindows, pattern_index );
	//cout << " before cover: " << endl;
	//print_patterns_and_mwindows( sorted_list_of_mwindows );
	bool covered_pos[ D->getSize() ] = { false };

	/*cout << "Complete list: " << endl;
			print_patterns_and_mwindows( sorted_list_of_mwindows );*/

	if( new_candidate == nullptr || selected_windows.size() == 0 )
	{
		selected_windows.clear();
		choose_windows( 0, sorted_list_of_mwindows, selected_windows, D, covered_pos, 0, D->getSize() );
	}
	else
	{
		//cout << "new candidate sort pos: " << new_candidate->getSortIndex() << " / " << list_of_patterns.size() << endl;
		//cout << "Num of windows: " << sorted_list_of_mwindows[new_candidate->getSortIndex()].size() << endl;
		reuse_previous_cover_and_choose_windows( selected_windows, sorted_list_of_mwindows, new_candidate, D, D->getSize());
	}
	//cout << "Before calc code lengh: " << endl;

	new_code_len = calc_code_len( selected_windows, list_of_patterns, D, sorted_list_of_mwindows, new_susages, list_of_choice_patterns, prune_mode );
	//cout << "After calc code lengh. " << endl;

	return 0;
}
