/*
 * ChoicePattern.cpp
 *
 *  Created on: Nov 28, 2015
 *      Author: abhattac
 */

#include "ChoicePattern.h"
#include "Pattern.h"
#include "EncodingCalucator.h"
#include "Data.h"
#include "LabelList.h"
#include <climits>
#include <cstdint>

using namespace std;

typedef uint32_t uint32;
typedef int32_t int32;

ChoicePattern::ChoicePattern(Pattern *first_pattern, Data *D) {
	// TODO Auto-generated constructor stub
	this->point_of_diff = -1;
	this->sub_patterns.push_back( first_pattern );
	this->codelen = get_codelen( D );
}

double ChoicePattern::get_codelen( Data *D )
{
	if( this->codelen == INT_MAX )
	{
		EncodingCalucator ec;
		double tmp_codelen = 0;
		if( this->point_of_diff == -1 && this->sub_patterns.size() > 1 )
		{
			cout << "Error in getCodelen! sub_pattern size: " << sub_patterns.size() << endl;
			this->print();
			this->sub_patterns.pop_back();
			return INT_MAX;
		}
		else if( this->point_of_diff == -1 && this->sub_patterns.size() == 1 )
		{
			Pattern *first_pattern = this->sub_patterns[0];
			tmp_codelen += ec.ln( first_pattern->getSize() );
			for(uint32 j = 0; j < first_pattern->getSize(); j++ )
			{
				tmp_codelen += ec.lu( (double)D->getSize() / D->getAlphabetSupport( first_pattern->charAt(j)) );
			}
		}
		else
		{
			int num_of_spots = this->sub_patterns[0]->getSize() + this->sub_patterns.size() - 1;
			tmp_codelen += ec.ln( num_of_spots ) + ec.lu( num_of_spots - 1 );
			tmp_codelen += ec.approxlogchoose( num_of_spots - 1, this->sub_patterns.size() - 1 );
			for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
			{
				tmp_codelen += ec.lu( (double)D->getSize() / D->getAlphabetSupport( this->sub_patterns[0]->charAt(j)) );
			}
			for(uint32 i = 1; i < this->sub_patterns.size(); i++ )
			{
				tmp_codelen += ec.lu( (double)D->getSize() / D->getAlphabetSupport( this->sub_patterns[i]->charAt( this->point_of_diff )) );
			}
		}
		this->codelen = tmp_codelen;
		return tmp_codelen;
	}
	else
	{
		return this->codelen;
	}
}

double ChoicePattern::change_in_codelen_if_added( Pattern *new_pattern, Data *D )
{
	bool same_flag = true;
	for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
	{
		if( new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
		{
			same_flag = false;
		}
	}
	if( same_flag )
	{
		return INT_MAX;
	}

	int32 tpoint_of_diff = - 1;
	if( this->sub_patterns[0]->getSize() != new_pattern->getSize() )
	{
		return INT_MAX;
	}
	else if( this->point_of_diff == -1 )
	{
		uint32 tpoints_of_diff = 0;
		for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				tpoints_of_diff++;
				tpoint_of_diff = j;
			}
		}
		if( tpoints_of_diff > 1 )
		{
			return INT_MAX;
		}
		//cout << "Points of diff: " << tpoints_of_diff << endl;
	}
	else
	{
		for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( j != this->point_of_diff && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				return INT_MAX;
			}
		}
		tpoint_of_diff = this->point_of_diff;
	}
	this->sub_patterns.push_back( new_pattern );
	double old_codelen = this->codelen;
	this->codelen = INT_MAX;
	int32 old_point_of_diff = this->point_of_diff;
	this->point_of_diff = tpoint_of_diff;
	double new_codelen = get_codelen( D );
	this->sub_patterns.pop_back();
	this->codelen = INT_MAX;
	this->point_of_diff = old_point_of_diff;
	get_codelen( D );
	return (old_codelen - new_codelen);
}

bool ChoicePattern::pattern_can_be_added( Pattern *new_pattern )
{
	bool same_flag = true;
	for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
	{
		if( new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
		{
			same_flag = false;
		}
	}
	if( same_flag )
	{
		return false;
	}
	if( this->point_of_diff == -1 )
	{
		bool flag = false;
		for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( !flag && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				flag = true;
			}
			else if( flag && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				return false;
			}
		}
	}
	else
	{
		if( new_pattern->getSize() != this->sub_patterns[0]->getSize() )
		{
			return false;
		}
		if( new_pattern->charAt( this->point_of_diff ) != this->sub_patterns[0]->charAt( this->point_of_diff ) )
		{
			return false;
		}
		for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( j != this->point_of_diff && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				return false;
			}
		}
	}
	return true;
}

void ChoicePattern::add_pattern( Pattern *new_pattern, Data *D )
{
	if( this->point_of_diff == -1 )
	{
		bool flag = false;
		int32 tpoint_of_diff = -1;
		for(uint32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( !flag && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				flag = true;
				tpoint_of_diff = j;
			}
			else if( flag && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				cout << "Error in add_pattern 1!" << endl;
				exit(0);
			}
		}
		this->point_of_diff = tpoint_of_diff;
	}
	else
	{
		for(int32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
		{
			if( j != this->point_of_diff && new_pattern->charAt(j) != this->sub_patterns[0]->charAt(j) )
			{
				cout << "Error in add_pattern 2!" << endl;
				exit(0);
			}
		}
	}
	this->sub_patterns.push_back( new_pattern );
	this->codelen = INT_MAX;
	this->codelen = get_codelen( D );
}

int ChoicePattern::index_of_sub_pattern( Pattern *query_pattern )
{
	for(uint32 i = 0; i < this->sub_patterns.size(); i++ )
	{
		if( this->sub_patterns[i]->getPatternNum() == query_pattern->getPatternNum() )
		{
			return i;
		}
	}
	return -1;
}

void ChoicePattern::delete_sub_pattern( uint32_t idx, Data *D )
{
	this->sub_patterns.erase( this->sub_patterns.begin() + idx );
	this->codelen = INT_MAX;
	this->get_codelen( D );
}

void ChoicePattern::print()
{
	for(int32 j = 0; j < this->sub_patterns[0]->getSize(); j++ )
	{
		if( j != this->point_of_diff )
		{
			cout << this->sub_patterns[0]->charAt( j ) << " - ";
		}
		else
		{
			cout << " - {";
			for(uint32 k = 0; k < this->sub_patterns.size(); k++ )
			{
				cout << this->sub_patterns[k]->charAt( this->point_of_diff ) << " | ";
			}
			cout << " - }";
		}
	}
	cout << endl;
}

void ChoicePattern::print(LabelList &label_list)
{
	for(int j = 0; j < this->sub_patterns[0]->getSize(); j++ )
	{
		if( j != this->point_of_diff )
		{
			cout << label_list.get_label( this->sub_patterns[0]->charAt( j ) ) << " - ";
		}
		else
		{
			cout << " - {";
			for(int k = 0; k < this->sub_patterns.size(); k++ )
			{
				cout << label_list.get_label( this->sub_patterns[k]->charAt( this->point_of_diff ) ) << " | ";
			}
			cout << " - }";
		}
	}
	cout << endl;
}

Pattern* ChoicePattern::get_sub_pattern( uint32_t idx )
{
	return this->sub_patterns[ idx ];
}

int ChoicePattern::get_num_of_sub_patterns()
{
	return this->sub_patterns.size();
}

ChoicePattern::~ChoicePattern() {
	// TODO Auto-generated destructor stub

} /* namespace std */
