/*
 * Pattern.cpp
 *
 *  Created on: Mar 10, 2015
 *      Author: abhattac
 */

#include "../misc/Pattern.h"
#include "../misc/EncodingCalucator.h"

#include <string.h>
#include <math.h>
#include <iostream>
#include <stdio.h>
#include <cstdint>

using namespace std;

Pattern::Pattern(vector<int32_t> &p)
{
	// TODO Auto-generated constructor stub
	this->usage = 0;
	this->gaps = 0;
	this->pattern_code_length = 0;
	this->preq_fill_code_length = 0;
	this->preq_gap_code_length = 0;
	this->p.reserve( p.size() );

	for(int32_t i = 0; i < p.size(); i++)
	{
		this->p.push_back( p[i] );
	}
	this->size = p.size();
	this->num = 0;
	this->stlen = 0;
	this->max_gap_len = 0;
	this->hist.reserve(1);
	this->hist.push_back(0);
	this->hist_size = 1;
	this->merge_pos = 0;
}

Pattern::Pattern(vector<int32_t> &p, int32_t merge_pos)
{
	// TODO Auto-generated constructor stub
	this->usage = 0;
	this->gaps = 0;
	this->pattern_code_length = 0;
	this->preq_fill_code_length = 0;
	this->preq_gap_code_length = 0;
	this->p.reserve( p.size() );

	for(int32_t i = 0; i < p.size(); i++)
	{
		this->p.push_back( p[i] );
	}
	this->size = p.size();
	this->num = 0;
	this->stlen = 0;
	this->max_gap_len = 0;
	this->hist.reserve(1);
	this->hist.push_back(0);
	this->hist_size = 1;
	this->merge_pos = merge_pos;
}

int32_t Pattern::charAt(int i) const {
	if( i < size )
		return p[i];
	else
		return 0;
}

int32_t Pattern::charAt(uint32_t i) const {
	if( i < size )
		return p[i];
	else
		return 0;
}


void Pattern::setPatternNum(int32_t num) {
	this->num = num;
}

uint32_t Pattern::getSize() const {
	return size;
}

void Pattern::setGaps(int32_t gaps) {
	this->gaps = gaps;
}

void Pattern::setUsage(int32_t usage) {
	this->usage = usage;
}

uint32_t Pattern::getUsage() const {
	return this->usage;
}

uint32_t Pattern::getGaps() const {
	return this->gaps;
}

void Pattern::setPatternCodeLength(double len) {
	this->pattern_code_length = len;
}

double Pattern::getGapCodeLength() const {
	if( this->gaps == 0 /*|| this->usage == 0*/ )
	{
		return 0;
	}
	double fills = this->usage * (this->size -1);
	return log2((double)(this->gaps + fills)/(this->gaps));
}

double Pattern::getFillCodeLength() const {
	if( this->gaps == 0 /*|| this->usage == 0*/ )
	{
		return 0;
	}
	double fills = this->usage * (this->size -1);
	return log2((double)(this->gaps + fills)/(fills));
}

double Pattern::getPatternCodeLength() const {
	/*if( this->usage == 0 )
	{
		return 0;
	}*/
	return this->pattern_code_length;
}

void Pattern::setPatternStLength(double len) {
	this->stlen = len;
}

double Pattern::getPatternStLength() const {
	return stlen;
}

void Pattern::update_hist(int32_t gap_len) {
	//cout << "debug 4" << endl;
	if( gap_len > max_gap_len )
	{
		for(int32_t i = max_gap_len + 1 ; i <= gap_len; i++)
		{
			if( i >= hist_size )
			{

				hist.reserve( 2 * hist_size );
				for(int32_t j = 0; j < hist_size; j++ )
				{
					hist.push_back(0);
				}
				hist_size *= 2;
			}
			hist[i] = 0;
		}
		max_gap_len = gap_len;
	}
	hist[gap_len] = hist[gap_len] + 1;
	//cout << "pos: " << gap_len << " - " << hist[gap_len] << endl;
}

void Pattern::reset_hist() {
	max_gap_len = 0;
	hist.clear();
	hist.reserve(1);
	hist_size = 1;
}

double Pattern::getGapCodeLength(int32_t gap_size) const {
	EncodingCalucator ec;
	return (ec.lu( (double) usage / hist[gap_size] ) + ec.logWeakNumberComposition( gap_size, size - 1));
}

double Pattern::getMaxGap() const {
	return max_gap_len;
}

uint32_t Pattern::getPatternNum() const {
	return this->num;
}

void Pattern::printPattern()
{
	for(int32_t j = 0; j < this->p.size(); j++ )
	{
		cout << this->p[j] << " - ";
	}
	cout << endl;
}

double Pattern::getPreqGapCodeLength() const
{
	return this->preq_gap_code_length;
}
double Pattern::getPreqFillCodeLength() const
{
	return this->preq_fill_code_length;
}

void Pattern::setPreqGapCodeLength(double pgcl)
{
	this->preq_gap_code_length = pgcl;
}

void Pattern::setPreqFillCodeLength(double pfcl)
{
	this->preq_fill_code_length = pfcl;
}

uint32_t Pattern::getMergePos() const
{
	return this->merge_pos;
}

uint32_t Pattern::getSortIndex() const
{
	return this->sort_index;
}

void Pattern::setSortIndex(int32_t index)
{
	this->sort_index = index;
}

Pattern::~Pattern() {
	// TODO Auto-generated destructor stub
	p.clear();
	hist.clear();
}

double Pattern::getGainInCodeLength() const
{
	return this->gain_in_code_len;
}

double Pattern::setGainInCodeLength( double gain )
{
	this->gain_in_code_len = gain;
}

void Pattern::setStops( uint32_t stops )
{
	this->stops = stops;
}

uint32_t Pattern::getStops()
{
	return this->stops;
}
