/*
 * create_data.cpp
 *
 *  Created on: Aug 25, 2015
 *      Author: abhattac
 */
#include "Pattern.h"
#include <vector>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <locale>
#include <sstream>
#include <string>
#include <algorithm>
#include <chrono>
#include <random>

using namespace std;

Pattern *new_pattern(int length_of_pattern)
{
	vector<int> p_alpha;
	for(int i = 0; i < length_of_pattern; i++)
	{
		p_alpha.push_back( rand() % 1000 );
	}
	Pattern *newp = new Pattern( p_alpha );
	return newp;
}

void create_patterns(int num_of_patterns, int length_of_pattern, vector<Pattern*> &list_of_patterns)
{
	for(int  i = 0; i < num_of_patterns; i++)
	{
		list_of_patterns.push_back( new_pattern( length_of_pattern ) );
	}
}

void init_random_data(vector<int> &data, int length_of_data)
{
	for(int  i = 0; i < length_of_data; i++)
	{
		data.push_back( i / 10 );
	}

	unsigned seed = std::chrono::system_clock::now().time_since_epoch().count();

	shuffle ( data.begin(), data.end(), default_random_engine(seed) );
}

bool toss_coin(double prob_of_success)
{
	double res = rand() % 100;
	if( res <= ( prob_of_success * 100 ) )
	{
		return true;
	}
	return false;
}

void insert_pattern_at_pos(int pos, Pattern *pattern, vector<int> &data, double prob_of_gap)
{
	int count  = 0;
	while( count < pattern->getSize() )
	{
		data[ pos ] = pattern->charAt( count );
		count ++;
		if( toss_coin( prob_of_gap ) )
		{
			pos += 2;
		}
		else
		{
			pos++;
		}
	}
}

void dfa_1_nest_patterns(int pos, Pattern *p1, Pattern *p2, vector<int> &data, double prob_of_gap)
{
	if( toss_coin( 0.5 ) )
	{
		Pattern* tmp = p1;
		p1 = p2;
		p2 = tmp;
	}

	int count  = 0;
	bool flag = false;
	while( count < p1->getSize() )
	{
		data[ pos ] = p1->charAt( count );
		count ++;
		if( toss_coin( prob_of_gap ) )
		{
			pos += 2;
		}
		else
		{
			pos++;
		}
		if( toss_coin( 0.5 ) && !flag )
		{
			flag = true;
			insert_pattern_at_pos( pos, p2, data, prob_of_gap);
		}
	}

	if( !flag )
	{
		insert_pattern_at_pos( pos, p2, data, prob_of_gap);
	}

}

void insert_overlapping_patterns_at_pos(int pos, Pattern *p1, Pattern *p2, vector<int> &data, double prob_of_gap)
{
	int count_p1  = 0, count_p2 = 0;
	while( count_p1 < p1->getSize() || count_p2 < p2->getSize() )
	{
		if( count_p1 < p1->getSize() && toss_coin( 0.8 ) )
		{
			data[ pos ] = p1->charAt( count_p1 );
			count_p1 ++;
		}
		else
		{
			data[ pos ] = p2->charAt( count_p2 );
			count_p2 ++;
		}
		if( toss_coin( prob_of_gap ) )
		{
			pos += 2;
		}
		else
		{
			pos++;
		}
	}
}


void write_patterns_to_file(vector<Pattern*> list_of_patterns, string filename)
{
	ofstream pattern_file ( filename );

	for(unsigned int i = 0; i < list_of_patterns.size(); i++)
	{
		string p_alpha;

		for(int  j = 0; j < list_of_patterns[i]->getSize(); j++)
		{
			int event = list_of_patterns[i]->charAt( j );

			string sevent;

			ostringstream convert;

			convert << event;

			sevent = convert.str();

			p_alpha += sevent + ",";
		}

		pattern_file << p_alpha;
	}

	pattern_file.close();
}

void write_data_to_file(vector<int> &data, string filename)
{
	ofstream data_file ( filename );

	string data_alpha;

	for(unsigned int i = 0; i < data.size(); i++)
	{
		int event = data[i];

		string sevent;

		ostringstream convert;

		convert << event;

		sevent = convert.str();

		data_alpha += sevent + ",";

	}

	data_file << data_alpha;

	data_file.close();
}

void insert_patterns_normally( vector<Pattern*> &list_of_patterns, vector<int> &data, int num_of_times_to_plant, double prob_of_gap )
{
	for(unsigned int i = 0; i < list_of_patterns.size(); i++ )
	{
		for( int j = 0; j < num_of_times_to_plant; j++ )
		{
			insert_pattern_at_pos( rand() % 9995, list_of_patterns[i], data, prob_of_gap);
		}
	}
}

void insert_patterns_as_nested_pairs( vector<Pattern*> &list_of_patterns, vector<int> &data, int num_of_times_to_plant, double prob_of_gap )
{
	for(unsigned int i = 0; i < list_of_patterns.size(); i += 2 )
	{
		for( int j = 0; j < num_of_times_to_plant; j++ )
		{
			dfa_1_nest_patterns( rand() % 9995, list_of_patterns[i] , list_of_patterns[i + 1] , data, prob_of_gap);
		}
	}
}

void insert_patterns_as_overlapping_pairs( vector<Pattern*> &list_of_patterns, vector<int> &data, int num_of_times_to_plant, double prob_of_gap )
{
	for(unsigned int i = 0; i < list_of_patterns.size(); i += 2 )
	{
		for( int j = 0; j < num_of_times_to_plant; j++ )
		{
			insert_overlapping_patterns_at_pos( rand() % 9995, list_of_patterns[i] , list_of_patterns[i + 1] , data, prob_of_gap);
		}
	}
}

void create_data(int num_of_patterns, int length_of_patterns,
		string pattern_filename, int num_of_times_to_plant, double prob_of_gap, string data_filename, bool overlapping)
{
	vector<Pattern*> list_of_patterns;

	int tmp_vec1[] = {1,2,3,4,5};
	vector<int> p1_alpha (tmp_vec1, tmp_vec1 + sizeof(tmp_vec1) / sizeof(int) );
	Pattern *p1 = new Pattern( p1_alpha );
	list_of_patterns.push_back( p1 );

	int tmp_vec2[] = {1,2,6,4,5};
	vector<int> p2_alpha (tmp_vec2, tmp_vec2 + sizeof(tmp_vec2) / sizeof(int) );
	Pattern *p2 = new Pattern( p2_alpha );
	list_of_patterns.push_back( p2 );

	int tmp_vec3[] = {1,2,7,4,5};
	vector<int> p3_alpha (tmp_vec3, tmp_vec3 + sizeof(tmp_vec3) / sizeof(int) );
	Pattern *p3 = new Pattern( p3_alpha );
	list_of_patterns.push_back( p3 );

	vector<int> data;
	init_random_data( data, 10000);

	insert_patterns_normally( list_of_patterns, data, 10, 0.1 );

	write_patterns_to_file( list_of_patterns, pattern_filename);
	write_data_to_file( data, data_filename);

}


