# -*- coding: utf-8 -*-

import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

with open('requirements.txt', 'r') as f:
    install_reqs = [
        s for s in [
            line.split('#', 1)[0].strip(' \t\n') for line in f
        ] if s != ''
    ]

with open('version.txt', 'r') as f:
    version = f.read().strip()

# This call to setup() does all the work
setup(
    name="shs-plot-utils",
    version=version,
    description="plot utils for faster creation of plots in our projects",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.dillinger.de/KI/DataScience/utils/shs-plot-utils",
    author="Benjamin Blaß, Nadja Wirth, Boris Wiegand",
    author_email="benjamin.blass@dillinger.biz, nadja.wirth@dillinger.biz, boris.wiegand@dillinger.biz",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: matplotlib",
        "Topic :: plot",
        "Topic :: plotting",
        "Topic :: plots",
        "Topic :: graphs",
        "Topic :: diagrams",
    ],
    packages=["shsplots"],
    include_package_data=True,
    package_data={
            "shsplots": ['*','*/*','*/*/*','*/*/*/*']
    },
    exclude_package_data={
            "": ['*.pyc']
    },
    install_requires=install_reqs
)
