# -*- coding: utf-8 -*-

import os
import unittest
from unittest.mock import patch
import tempfile
import matplotlib.pyplot as plt
from shsplots.matplotlib.plot_context import PlotContext

class TestPlotContext(unittest.TestCase):

    def test_plot_context_show(self):
        with patch('matplotlib.pyplot.show') as mocked_show:
            with PlotContext():
                plt.plot([1,2,3], [1,2,3])
            mocked_show.assert_called_once()

    def test_plot_context_filesave_only(self):
        with patch('matplotlib.pyplot.show') as mocked_show:
            with tempfile.TemporaryDirectory() as directory:
                path_to_plot = os.path.join(directory, 'plot.png')
                with PlotContext(show=False, filepath=path_to_plot):
                    plt.plot([1,2,3], [1,2,3])
                mocked_show.assert_not_called()
                self.assertTrue(os.path.exists(path_to_plot))

    def test_use_tight_layout(self):
        with patch('matplotlib.pyplot.tight_layout') as mock:
            with PlotContext(show=False, use_tight_layout=False):
                plt.plot([1,2,3], [1,2,3])
            mock.assert_not_called()
            with PlotContext(show=False, use_tight_layout=True):
                plt.plot([1,2,3], [1,2,3])
            mock.assert_called_once()

    def test_to_base_64(self):
        with PlotContext(show=False, use_tight_layout=False) as plot_context:
            plt.plot([1,2,3], [1,2,3])
            self.assertIsInstance(plot_context.to_base_64(), str)

if __name__ == '__main__':
    unittest.main()
